/* PipeWire */
/* SPDX-FileCopyrightText: Copyright © 2019 Wim Taymans */
/* SPDX-License-Identifier: MIT */

#include <dbus/dbus.h>

#include <spa/support/dbus.h>
#include <spa-private/dbus-helpers.h>

#include "log.h"
#include "dbus-name.h"

void dbus_request_name(struct spa_dbus_connection *conn, const char *name)
{
	DBusConnection *bus = spa_dbus_connection_get(conn);
	spa_auto(DBusError) error = DBUS_ERROR_INIT;

	if (bus == NULL)
		return;

	if (dbus_bus_request_name(bus, name,
			DBUS_NAME_FLAG_DO_NOT_QUEUE,
			&error) == DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER)
		return;

	if (dbus_error_is_set(&error))
		pw_log_error("Failed to acquire %s: %s: %s", name, error.name, error.message);
	else
		pw_log_error("D-Bus name %s already taken.", name);
}
