/* PipeWire */
/* SPDX-FileCopyrightText: Copyright © 2025 Myrrh Periwinkle */
/* SPDX-License-Identifier: MIT */

#include <errno.h>

#include <dbus/dbus.h>

#include <spa/support/dbus.h>
#include <spa/support/plugin.h>
#include <spa-private/dbus-helpers.h>

#include "log.h"
#include "dbus-permission-store.h"

bool dbus_check_recording_permission(struct spa_dbus_connection *conn, const char *app_id)
{
	DBusConnection *bus;
	spa_autoptr(DBusMessage) m = NULL, r = NULL;
	spa_auto(DBusError) error = DBUS_ERROR_INIT;
	DBusMessageIter iter, sub;

	if (app_id == NULL)
		return false;

	if (conn == NULL)
		return false;

	bus = spa_dbus_connection_get(conn);
	if (bus == NULL)
		return false;

	m = dbus_message_new_method_call("org.freedesktop.impl.portal.PermissionStore",
					 "/org/freedesktop/impl/portal/PermissionStore",
					 "org.freedesktop.impl.portal.PermissionStore",
					 "Lookup");
	if (m == NULL)
		return false;

	if (!dbus_message_append_args(m,
				      DBUS_TYPE_STRING, &(const char *){ "devices" },
				      DBUS_TYPE_STRING, &(const char *){ "microphone" },
				      DBUS_TYPE_INVALID))
		return false;

	r = dbus_connection_send_with_reply_and_block(bus, m, -1, &error);
	if (r == NULL || dbus_message_get_type(r) != DBUS_MESSAGE_TYPE_METHOD_RETURN)
		return false;

	dbus_message_iter_init(r, &iter);
	if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_ARRAY)
		return false;

	dbus_message_iter_recurse(&iter, &sub);
	while (dbus_message_iter_get_arg_type(&sub) == DBUS_TYPE_DICT_ENTRY) {
		DBusMessageIter entry, value;
		char *entry_app_id;

		dbus_message_iter_recurse(&sub, &entry);
		if (dbus_message_iter_get_arg_type(&entry) != DBUS_TYPE_STRING)
			goto next;

		dbus_message_iter_get_basic(&entry, &entry_app_id);
		if (strcmp(entry_app_id, app_id) != 0)
			goto next;

		dbus_message_iter_next(&entry);
		if (dbus_message_iter_get_arg_type(&entry) != DBUS_TYPE_ARRAY)
			goto next;

		dbus_message_iter_recurse(&entry, &value);
		while (dbus_message_iter_get_arg_type(&value) == DBUS_TYPE_STRING) {
			char *permission;

			dbus_message_iter_get_basic(&value, &permission);
			if (strcmp(permission, "yes") == 0)
				return true;

			dbus_message_iter_next(&value);
		}

next:
		dbus_message_iter_next(&sub);
	}

	return false;
}
